from typing import List
import logging
import re
from subprocess import run, PIPE
from datetime import timedelta

from ruxit.api.base_plugin import BasePlugin
from ruxit.api.selectors import HostSelector

log = logging.getLogger(__name__)


class ADReplicationExtension(BasePlugin):
    def initialize(self, **kwargs):
        self.executions = 0

    def query(self, **kwargs):

        frequency = int(self.config.get("frequency", 1))
        log.setLevel(self.config.get("log_level", "INFO"))
        share_names = self.config.get("share_names").split(",")
        share_hosts = self.config.get("share_hosts").split(",")

        if self.executions % frequency == 0:

            try:
                cmd = run("repadmin /replsummary", stdin=PIPE, stdout=PIPE, universal_newlines=True)
                log.error(f"repadmin - stderr: '{cmd.stderr}'")
                lines = cmd.stdout.splitlines()
                log.debug(f"repadmin - lines: {lines}")
                self.parse_repadmin_lines(lines)

            except Exception as e:
                log.exception(f"repadmin - error running command: {e}")

            for share_host in share_hosts:
                try:
                    cmd = run(f"net view {share_host}", stdin=PIPE, stdout=PIPE, universal_newlines=True)
                    lines = cmd.stdout.splitlines()
                    log.error(f"net view - stderr: '{cmd.stderr}'")
                    log.debug(f"net view - lines: {lines}")

                    for share_name in share_names:
                        available = check_if_mounted(share_name, lines)
                        log.debug(f"Shared resource '{share_name}' is {'NOT ' if not available else ''}available")
                        self.results_builder.absolute(
                            key="shared_available",
                            value=100 if available else 0,
                            dimensions={"Name": share_name, "Host": share_host},
                            entity_selector=HostSelector(),
                        )
                except Exception as e:
                    log.exception(f"net view - error running command: {e}")
            self.executions += 1

    def parse_repadmin_lines(self, lines: List[str]):
        capturing = None
        for line in lines:
            if "source dsa" in line.lower():
                capturing = "source"
                continue
            if "destination dsa" in line.lower():
                capturing = "destination"
                continue

            if capturing and line:
                fields = line.split(maxsplit=6)
                fields.append(None) if len(fields) == 6 else None
                if len(fields) == 7:
                    server, raw_delta, fails, _, total, pct, error = fields
                    delta = parse_time(raw_delta)

                    log.debug(f"{server}, {delta}, {fails}, {error}")
                    self.results_builder.absolute(
                        key=f"{capturing}_delta",
                        value=delta.total_seconds(),
                        dimensions={"Server": server},
                        entity_selector=HostSelector(),
                    )
                    self.results_builder.absolute(
                        key=f"{capturing}_errors", value=fails, dimensions={"Server": server}, entity_selector=HostSelector(),
                    )


def check_if_mounted(share_name: str, lines: List[str]):
    for line in lines:
        if share_name.lower() in line.lower():
            return True
    return False


def parse_time(raw_time: str) -> timedelta:
    # 19h:41m:19s -> 19:41:19
    time_without_digits = re.sub(r"[a-z]", "", raw_time.lower())[-11:]

    while len(time_without_digits) < 11:
        # 19:41:19 -> 00:19:41:19
        time_without_digits = f"00:{time_without_digits}"

    days, hours, minutes, seconds = [int(t) for t in time_without_digits.split(":")]
    return timedelta(days=days, hours=hours, minutes=minutes, seconds=seconds)
